/* 
 * Copyright (C) 2013 Rafael Barragán Górriz
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Class Board Manager
 * Provides methods to load, sync and edit boards
 * @type Object
 */
var bm = {
    
    server: {'getBoard': 'board/get/',
             'getCategories': 'categories/get/',
             'getSymbols': 'symbols/get/'
            },
    
    local: {'board': 'board.json',
            'categories': 'categories.json',
            'symbols': 'symbols.json'
           },
           
    dbURI: '',     // URI to used db. This will be setted by getBoard function and used in View objects
    
    localBasePath: 'index.html',
    
    localForms: {
        'board': '#board',
    },
   
    /**
     * Class initializer
     * @returns {undefined}
     */
    initialize: function () {
        // Add router handlers [
        var routes = {};
        
        // Add routes
        // routes[bm.localForms.board]
        routes[bm.localForms.board] = { events: "s", 
                                        handler: bm.boardOnShow
                                        };
        
        app.router.add( routes );
        
        // Prepend localBasePath to localforms
        for (key in bm.localForms) {
            bm.localForms[key] = bm.localBasePath + bm.localForms[key];
        }
    },
    
    /**
     * Event handler for OnShow Event
     * @param {type} eventType
     * @param {type} matchObj
     * @param {type} ui
     * @param {type} page
     * @param {type} evt
     * @returns {undefined}
     */
    boardOnShow: function (eventType, matchObj, ui, page, evt) {
        console.log('BoardOnShow');
        //$(page).on("click", ".edit-board-button",bm.editBoard );
    },
    
    /**
     * Enable board edition
     * @returns {}
     */
    enableEdit: function () {
         if (confirm('Activar el modo edición')) {
             bm.view.renderEdit();
             bm.board.selector.stop();
         }
    },
    
    /**
     * Disable board edition
     * @returns {Boolean}
     */
    disableEdit: function () {
        // TODO: check canges in board and ask user to save
        if (confirm('Salir del modo de edición?')) {
            bm.board.selector.play();
            return true;
        }
        return false;
    },
    
    /**
     * Exit board
     * @returns {Boolean}
     */
    exit: function () {
        if (confirm('Seguro que quieres salir del tablero?')) {
            app.closeBoardAction();
            return true;
        }
        return false;
    },
    
    /**
     * Load the board associated to the profile
     * @param {Object} profile
     * @returns {promise}
     */
    loadBoardAction: function (profile) {
        app.loadingMessage('Cargando tablero...');
        bm.getBoard(profile.key)
            .done(function (board) {
                    console.log('Board getted ...');
                    bm.board = board;
                    // TODO: Load selector setted on board conf
                    bm.board.selector = new DirectSelector();
                    console.log('Creating board view ...');
                    bm.view = new BoardView(_.extend(bm.board,{ el: $("#board")}));
                    app.loadLocalPage(bm.localForms.board);
                })
            .fail(function (error) {
                console.log(arguments);
                app.alertMessage(error);
            });
    },
    
    /**
     * Get Board Object. In case of mobile, synchronise action will be attempted.
     * @param {String} userKey
     * @param (Boolean) withOutSync Optional. Default false. If true, no sync will be attempted
     * @returns {Promise} Promise object
     */
    getBoard: function (userKey, withOutSync) {
        console.log('Get Board ...');
        if (app.isCordova) {
            console.log('Is Cordova ...');
            if (app.isConnected()) {
                console.log('Is connected ...');
                if (! withOutSync) {
                    console.log('Synch enabled.');
                    return bm.syncBoard(userKey);
                } else {
                    console.log('Get from local ...');
                    return bm.getBoardFromLocal(userKey);
                }
            } else {
                console.log('Not connected ...');
                return bm.getBoardFromLocal(userKey);
            }
        } else {
            console.log('Not mobile. Get from remote ...');
            return bm.getBoardFromServer(userKey);
        }
    },
    
    /**
     * Get Board from local DB
     * @returns {Promise} Promise object
     */   
    getBoardFromLocal: function (userKey) {
        var deferred = $.Deferred();
        console.log('Getting board from local ...'); 
        $.when(bm.getFileFromLocalDB(userKey, bm.local.board),
               bm.getFileFromLocalDB(userKey, bm.local.categories),
               bm.getFileFromLocalDB(userKey, bm.local.symbols)
        ).done(function (board, categories, symbols) {
                console.log('Board, categories and symbols getted ...');
                bm.setDBURILocal(userKey);
                deferred.resolve(bm.assembleBoard($.parseJSON(board), 
                                                  $.parseJSON(categories), 
                                                  $.parseJSON(symbols)));
        })
        .fail(function (error) {
            console.log('Error ['+ error +'] getting board from local');
            deferred.reject(error);
        });
        return deferred.promise();
    },
    
    /**
     * Get board from server
     * @param {String} userKey
     * @returns {unresolved}
     */
    getBoardFromServer: function (userKey) {
        var deferred = $.Deferred();
        console.log('Getting board from server ...'); 
        $.when( app.serverRequest(bm.server.getBoard + userKey),
                app.serverRequest(bm.server.getCategories + userKey),
                app.serverRequest(bm.server.getSymbols + userKey)
        ).done(function (board, categories, symbols) {
            console.log('Board, categories and symbols getted ...');
            bm.setDBURIRemote(userKey);
            deferred.resolve(bm.assembleBoard(board[0], categories[0], symbols[0]));
        })
        .fail(function (jqXHR, textStatus, errorThrown) {
            var error = 'Error ['+ errorThrown +'] getting board';
            console.log(error);
            deferred.reject(error);
        });
        return deferred.promise();
    },
    
    setDBURILocal: function (userKey) {
        var fullPath = app.getLocalDBDir().fullPath;
        bm.dbURI = fullPath + '/' + userKey + '/';
        console.log('DBUri setted to: '+ bm.dbURI);
    },
    
    setDBURIRemote: function (userKey) {
        bm.dbURI = app.config.server + app.config.serverDBRoot + userKey + '/';
        console.log('DBUri setted to: '+ bm.dbURI);
    },
    
    /**
     * Assemble board pieces into a complete board object
     * @param {JSON} boardJSON
     * @param {JSON} categoriesJSON
     * @param {JSON} symbolsJSON
     * @returns {Board}
     */
    assembleBoard: function (boardJSON, categoriesJSON, symbolsJSON) {
         
        console.log('Assemnbling board ...');
        
        var board = new Board(boardJSON,{parse: true});
        var categories = new Categories(categoriesJSON,{parse: true});
        var symbols =  new Symbols(symbolsJSON,{parse: true});
        
        board.set('categories',categories);
        board.set('symbols',symbols);

        // Link blackBoard symbols objects
        _.each(['playSymbol', 'eraseSymbol', 'backSymbol'], function (symbol) {
            var blackBoard = board.get('blackBoard');
            blackBoard[symbol] = symbols.get(blackBoard.get(symbol));
        });

        // At load, selected category is defaultCategory
        board.selectedCategory = board.defaultCategory;

        // Link symbols and categories with categories
        var categoriesArray = categories.models;
        var nCat = categories.length;
        for (var i=0; i<nCat ;i++) {
            // Link category symbol
            var symbol = categoriesArray[i].get('symbol');
            categoriesArray[i].symbol = symbols.get(symbol.id);

            // Link selectable symbols
            var grid = categoriesArray[i].get('grid');
            var nGrid = grid.length;

            // Create a array of selectables
            var selectables = new Array();
            categoriesArray[i].set('selectables',selectables);

            for (var j=0;j<nGrid;j++) {

                if(! grid[j].model) {  // Empty symbol
                    selectables.push(null);
                } else if (grid[j].model == 'Category') {   // Category 
                    selectables.push(categories.get(grid[j].id));
                } else if (grid[j].model == 'Symbol') {     // Symbol
                    selectables.push(symbols.get(grid[j].id));
                }
            }
        };

        return board;
    },
    
    /**
     * Synchronize board
     * @param {String} userKey
     * @returns {Promise} Promise object
     */
    syncBoard: function (userKey) {
        var deferred = $.Deferred();
        app.loadingMessage('Sincronizando tablero...');
        console.log('Synchronizing ...');    
        console.log('Gettting local board ...'); 
        
        var syncRemote = function (local) {
            console.log('Gettting board from server ...'); 
            bm.getBoardFromServer(userKey)
                .done(function(remote) {
                    console.log('Remote board obtained.'); 
                    $.when(bm._syncBoards(userKey,local,remote))
                    .done(function() {
                        console.log('Board is synchronized ...');
                        bm.setDBURILocal(userKey);      // Set local uri
                        deferred.resolve(remote);    // Always return remote board because it will be always updated
                    })
                    .fail(function (error) {            // Fail synchronization
                        console.log("Error [" + error + "] synchronizing boards.");
                        deferred.reject(error);
                    });
                })
                .fail(function(error) {
                    console.log("Error [" + error + "] getting server board unable to synchronize.");
                    deferred.reject(error);
                });
        };
        
        bm.getBoardFromLocal(userKey)
            .done(function (local) {
                console.log('Local board obtained.'); 
                syncRemote(local);
            })
            .fail(function (error) {
                console.log('Local board not present. Creating a void model'); 
                syncRemote(new Board());
            });
        
        return deferred.promise();
    },
    
    /**
     * Synchornize boards
     * @param {Board} localBoard
     * @param {Board} remoteBoard
     * @returns {promises} Promise array
     */
    _syncBoards: function (userKey,localBoard, remoteBoard) {
        console.log('Cheking board versions ...');
        var promises = new Array();
        var localCategories = localBoard.get('categories');
        var localSymbols = localBoard.get('symbols');
        var remoteCategories = remoteBoard.get('categories');
        var remoteSymbols = remoteBoard.get('symbols');
        
        // Check board version
        if (localBoard.get('version') != remoteBoard.get('version')) {
            console.log('Change version detected on board. Local [' + localBoard.get('version')+ '] Remote ['+remoteBoard.get('version')+']');
            promises.push(bm.saveContentToLocalDB(userKey,JSON.stringify(remoteBoard.toJSON()),bm.local.board));
        } else {
            console.log('No changes in board detected.');
        }

        // Check categories version
        if (! localCategories || (localCategories.version != remoteCategories.version)) {
            if (! localCategories ) {
                console.log('No local categories detected. Getting from server ...');
            } else {
                console.log('Change version detected on categories. Local [' + localCategories.version + '] Remote ['+ remoteCategories.version +']');
            }
            
            promises.push(bm.saveContentToLocalDB(userKey,JSON.stringify(remoteCategories.toJSON()),bm.local.categories));
        } else {
            console.log('No changes in categories detected.');
        }

        // Check symbols version
        if (! localSymbols || (localSymbols.version != remoteSymbols.version)) {
            if (! localSymbols ) {
                console.log('No local symbols detected. Getting from server ...');
            } else {
                console.log('Change version detected on categories. Local [' + localSymbols.version + '] Remote ['+ remoteSymbols.version +']');
            }
            
            promises.push(bm.saveContentToLocalDB(userKey,JSON.stringify(remoteSymbols.toJSON()),bm.local.symbols));
            
            // Update media files
            var removeLocal = new Array();
            var addLocal = new Array();
            
            if (localSymbols) {
                console.log('Analizing local symbols ...');
                var localURIs = localSymbols.getURIs();
                console.log('Local uris:');
                console.log(localURIs);
                var remoteURIs = remoteSymbols.getURIs();
                console.log('Remote uris:');
                console.log(localURIs);
                removeLocal = _.difference(localURIs,remoteURIs);
                addLocal = _.difference(remoteURIs,localURIs);
            } else {
                console.log('No local symbols.');
                addLocal = remoteSymbols.getURIs();
            }

            // Remove unused files
            console.log('Removing '+removeLocal.length+' unused files.');
            for (var i=0;i<removeLocal.length; i++) {
                promises.push(bm.removeFileFromLocalDB(userKey,removeLocal[i]));
            }

            // Get new files
            console.log('Getting '+addLocal.length+' new files.');
            for (var i=0;i<addLocal.length; i++) {
                var serverPath = app.config.server + app.config.serverDBRoot + userKey + '/' + addLocal[i];
                console.log('Getting [' +serverPath+ '] from server.');
                promises.push(bm.saveFileToLocalDB(userKey,serverPath, addLocal[i]));
            }
        } else {
            console.log('No changes in symbols detected.');
        }
        return promises;
    },
    
    /**
     * Get a file from local DB 
     * @param {String} userKey
     * @param {String} file
     * @param {String} readType Optional. Default 'text'. Possible values (text,array,binary,url)
     * @returns {jqXHR} Promise Object
     */
    getFileFromLocalDB: function (userKey, file, readType) {
        console.log('Getting file ['+file+'] from user ['+userKey+'] db ...');
        var uri = app.getLocalDBDir().fullPath + '/' + userKey + '/' + file;
        return app.getLocalFileFromURI(uri,readType);
    },
    
    /**
     * 
     * @param {type} userKey
     * @param {type} uri
     * @param {type} destination
     * @param {type} params
     * @returns {Promise} Promise Object
     */
    saveFileToLocalDB: function (userKey, uri, destination, params) {
        
        var deferred = $.Deferred();
        console.log('Saving file ...');
        var path = app.getLocalDBDir().fullPath + '/' + userKey + '/' + destination;
        console.log('Destination file ['+path+'] ...');
        var ft = new FileTransfer();
        ft.download(uri, path, function (fileEntry) {
            console.log('File entry ['+fileEntry.fullPath+'] getted ...');
            deferred.resolve(fileEntry);
        }, function (fileTransferError) {
            var error = "Error [" + fileTransferError.code + "] downloading [" + fileTransferError.source + "] to [" + fileTransferError.target + "]";
            console.log(error);
            deferred.reject(error);
        });
        return deferred.promise();
    },
    
    /**
     * Remove a file from local db
     * @param {type} userKey    
     * @param {type} fileName
     * @param {type} params
     * @returns {Promise} Promise Object
     */
    removeFileFromLocalDB: function (userKey, fileName, params) {
        var deferred = $.Deferred();
        console.log('Removing local file ...');
        var filePath = userKey + '/' + fileName;
        console.log('File to remove ['+filePath+']');
        var fail = function (fileError) {
                        var error = 'Error ['+fileError.code+'] removing [' + filePath + ']';
                        console.log(error);
                        deferred.reject(error);
                    };
        app.getLocalDBDir().getFile(filePath, {create: false, exclusive: false},
            function (fileEntry) {
                console.log('File entry ['+fileEntry.fullPath+'] getted ...');
                fileEntry.remove(
                    function () {
                        console.log(filePath + ' removed.');
                        deferred.resolve();
                    },
                    fail);
            },
            fail
        );
       
        return deferred.promise();
    },
    
    /**
     * 
     * @param {String} userKey User
     * @param {String} content Content to write
     * @param {String} dest FileName destination
     * @returns {Promise} Promise Object
     */
    saveContentToLocalDB: function (userKey, content, dest) {
        var deferred = $.Deferred();
        console.log('Saving content to local DB ...');
        var path = userKey + '/' + dest;
        console.log('Saving content to ['+path+']');
        var fail = function (fileError) {
                        var error = "Error [" + fileError.code + "] saving [" + dest + "]";
                        console.log(error);
                        deferred.reject(error);
                    };
        app.getUserDBDir(userKey)
            .done( function (dir) {
                console.log('User dir obtained, creating file ...');
                dir.getFile(dest, {create: true}, 
                    function (fileEntry) {
                        console.log('File entry ['+fileEntry.fullPath+'] getted ...');
                        fileEntry.createWriter(
                            function (writer) {
                                console.log('Writer created ...');
                                writer.onwriteend = function () {
                                    console.log('Write end ...');
                                    deferred.resolve();
                                };
                                writer.onerror = function (fileError) {
                                    var error = "Error [" + fileError.code + "] writting [" + dest + "]";
                                    console.log(error);
                                    deferred.reject(fileError);
                                }
                                writer.write(content);
                            }, 
                            fail);
                    }, 
                    fail);
            })
            .fail(fail);
        
        return deferred.promise();
    },
}
//@source = boardManajer.js
