/* 
 * Copyright (C) 2013 Rafael Barragán Górriz
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Status constants
 */
var SELECTOR_STATUS =  {
    UNPREPARED:  0x00,      // Selector is not prepared to play, prepare call is required. This is the default state
    PREPARED:    0x01,      // Selector is prepared to play
    PLAYING:     0x02,      // Selector is playing now 
    STOPPED:     0x03,      // Selector is stopped
    PAUSED:      0x04,      // Selector is paused  
    CLEARED:     0x05       // Selector is cleard, there are no bounded elements
};

/**
 * Class Selector
 * @description It defines the interface selector. All the selectors must be child of this class
 * @type Selector
 */
var Selector = Backbone.Model.extend ({
    
    /**
     * CSS class constants 
     */ 
    CSS_CLASS: {                    
        FOCUS: 'focus',             // Focus className
        SELECTABLE: 'selectable'    // Selectable className
    },    
    
    __status: SELECTOR_STATUS.UNPREPARED,  // Status value
    
    /**
     * Selector string (jQuery Selector format) to define which selectable elements can be selected
     */
    selectorString: '',
    
    /**
     * Events to add for each selectable. Not be confused with object events.
     * Key will be event name on the selectable element
     * Value is the handler method. Methods are from this object
     */ 
    eventsList: {},
    
    __eventsListPrepared: [],     // Prepared array events

 /* Interface Methods
 ========================================================================== */     
    /**
     * prepare
     * @description Prepare selector to play
     *      On success status object will change to STATUS.PREPARED
     *      Redefine it to specific behavior
     * @param board Board to manage
     * @param options (Optional) Additional options
     * @return this 
     */
    prepare: function (board, options) {
        return this.prepareEvents(options).setStatus(SELECTOR_STATUS.PREPARED);
    },
    
    /**
     * play
     * @description Starts selector behavior. 
     *      On succes object state will change to STATUS.PLAYING
     * @param selector (Optional) jQuery selector string. It specify concret context to apply selector 
     * @return this
     */
    play: function (selector) {
        console.warn("Selector.play call. It's a abstract method, this call should not have happened.");
        return this;
    },
    
    /**
     * stop
     * @descripion Stop selector behavior. On resume it will start from first element
     *      On succes object state will change to STATUS.STOPPED
     * @params Nothing
     * @return this
     */
    stop: function () {
        console.warn("Selector.stop call. It's a abstract method, this call should not have happened.");
        return this;
    },
    
    /**
     * pause
     * @descripion Pause selector behavior. On resume it will continue from actual element
     *      On succes object state will change to STATUS.PAUSED
     * @params Nothing
     * @return this
     */
    pause: function () {
        console.warn("Selector.pause call. It's a abstract method, this call should not have happened.");
        return this;
    },
    
    /**
     * resume
     * @descripion Resume selector behavior. It will start depends on previous call (stop or pause)
     *      On succes object state will change to STATUS.PLAYING
     * @params Nothing
     * @return this
     */
    resume: function () {
        console.warn("Selector.resume call. It's a abstract method, this call should not have happened.");
        return this;
    },
    
    /**
     * rewind
     * @descripion Restart from first element. ShortHand to Stop an Resume
     *      On succes object state will change to STATUS.PLAYING
     * @params Nothing
     * @return this
     */
    rewind: function () {
        return this.stop().resume();
    },
    
    /**
     * clear
     * @descripion Clear list of elements to apply selector
     *      On succes object state will change to STATUS.CLEARED
     * @params Nothing
     * @return this
     */
    clear: function () {
        console.warn("Selector.clear call. It's a abstract method, this call should not have happened.");
        return this;
    },
    
/* Selector management
 ========================================================================== */     
    /**
     * setSelectorSring
     * @description Sets selector. 
     *      Stop, Clear and addEventsToSelection functions will be called before change.
     * @param selectorString String (jquery Selector format)
     * @return this
     */
    setSelectorSring: function (selectorString) {
        this.stop().clear().addEventsToSelection();    
        this.selectorString = selectorString;
        return this;
    },
    
    /**
     * getSelector
     * @description Get selector
     * @return String (jquery Selector format)
     */
    getSelectorSring: function () {
        return this.selectorString;
    },
    
    /**
     * getJqueryCompleteSelector
     * @description Get comlete selector include css selectable class and selector if specified.
     * @return String (jquery Selector format)
     */
    getJqueryCompleteSelector: function () {
        return ('.' + this.CSS_CLASS.SELECTABLE + ' ' + this.getSelectorSring()).trim();
    },
    
/* Events management
 ========================================================================== */    
    /**
     * prepare
     * @description Prepare events to bind to play. 
     *      On success status object will change to STATUS.PREPARED
     * @param options (Optional) Additional options
     * @return this 
     */
    prepareEvents: function (options) {

        this.__eventsListPrepared = new Array();

        // Prepare events 
        for (var key in this.eventsList) {

            // Parts of this code are extracted from backbone.js
            var method = this.eventsList[key];
            if (!_.isFunction(method)) method = this[this.eventsList[key]];
            if (!method) continue;

            // Cached regex to split keys for `delegate`.

            var delegateEventSplitter = /^(\S+)\s*(.*)$/;
            var match = key.match(delegateEventSplitter);
            var eventName = match[1], selector = match[2];

            this.__eventsListPrepared.push({
                                            "eventName": eventName,
                                            "method": method,
                                            "selector": selector});
        }

        return this;
     },
    
    /**
     * addEventsToSelection
     * @description Bind EventsList to selected elements
     * @return this
     */
    addEventsToSelection: function () {
        var selector = this.getJqueryCompleteSelector();
        console.log('[' + selector + '] elements will be in selectable elements list');

        var eventList = this.__eventsListPrepared; 
        
        for (var i=0;i < eventList.length; i++) {
            
            if (eventList[i].selector) {
                $( selector ).on(eventList[i].eventName,
                                 eventList[i].selector,
                                 _.bind(eventList[i].method,this));
            } else {
                $( selector ).on(eventList[i].eventName,
                                 _.bind(eventList[i].method,this));
            }
        }
        return this;
    },
    
    /**
     * removeEventsToSelection
     * @description UnBind EventsList from selected elements
     * @return this
     */
    removeEventsToSelection: function () {
        var selector = this.getJqueryCompleteSelector();
        console.log('[' + selector + '] elements will be removed of selectable elements list');
     
        var eventList = this.__eventsListPrepared; 
        
        for (var i=0;i < eventList.length; i++) {
            
            if (eventList[i].selector) {
                $( selector ).off(eventList[i].eventName,
                                 eventList[i].selector,
                                 _.bind(eventList[i].method,this));
            } else {
                $( selector ).off(eventList[i].eventName,
                                 _.bind(eventList[i].method,this));
            }
        }
        
        return this;
    },
       
    
/* Status management
 ========================================================================== */
    /**
     * isStatus
     * @description Return true if object status is status parameter
     * @param status Status mask
     * @return boolean
     */
    isStatus: function (status) {
        return this.__status & status;  
    },
    
    /**
     * setStatus
     * @description Set status to object status
     *      Previous states will not change 
     * @param status Status mask
     * @return this
     */
    setStatus: function (status) {
        this.__status = this.__status | status;
        return this;
    },
    
    /**
     * changeStatus
     * @description Change object status unsetting previous states
     * @param status Status mask
     * @return this
     */
    changeStatus: function (status) {
        this.__status = status;
        return this;
    },
    
    /**
     * unsetStatus
     * @description Unset status mask from object status
     * @param status Status mask
     * @return this
     */
    unsetStatus: function (status) {
        this.__status = this.__status & ~status;
        return this;
    }
    
});

//@ sourceURL=Selector.js