/* 
 * Copyright (C) 2013 Rafael Barragán Górriz
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Class Profile Manager
 * Provides method tu link, unlink, select, create, edit and erase users
 * @type Object
 */
var pm = {
    
    server: {'login': 'profile/login'},
    localBasePath: 'index.html',
    localForms: {
        'login': '#profile-login',
        'users': '#profile-users'
    },
    defaultUserKey: 'defaultUser',
    userListKey: 'users',
    
    /**
     * Class initializer
     * @returns {undefined}
     */
    initialize: function () {
        // Add router handlers [
        var routes = {};
        
        // Add routes
        routes[pm.localForms.login] = { 
                                    events: "s", 
                                    handler: pm.loginOnShow
                                    };
        routes[pm.localForms.users] = { 
                                    events: "s", 
                                    handler: pm.usersOnShow
                                    };                                        
        app.router.add( routes );
        
        // Prepend localBasePath to localforms
        for (key in pm.localForms) {
            pm.localForms[key] = pm.localBasePath + pm.localForms[key];
        }
    },
    
  
   /**
    * 
    * @param {type} eventType
    * @param {type} matchObj
    * @param {type} ui
    * @param {type} page
    * @param {type} evt
    * @returns {undefined}
    */
    loginOnShow: function (eventType, matchObj, ui, page, evt) {
        console.log('loginShow: ' + eventType);
        console.log(page);
        
        $(page).on("click", "button", function() {
            $(page).trigger('loginClick',evt);
        });
    },
    
    /*
     * 
     * @param {type} eventType
     * @param {type} matchObj
     * @param {type} ui
     * @param {type} page
     * @param {type} evt
     * @returns {undefined}
     */
    usersOnShow: function (eventType, matchObj, ui, page, evt) {
        console.log('usersload: ' + eventType);
        console.log(page);
        
        var list = $(page).find('ul');
        console.log(list);
        pm.populateUserList(list);
        list.listview( "refresh" );
        $(page).on("click", "button.edit-enable", pm.showLoginFormAction);
        $(page).on("click", "li", function() {
            $(page).trigger('listClick',this.dataset['key']);
        });
    },
    
    /**
     * Set default user
     */
    setDefaultUser: function (userKey) {
        localStorage[pm.defaultUserKey] = userKey;
    },
    
    /**
     * Return the default user key
     */
    getDefaultUser: function () {
        return pm.searchLocalUserLinked(localStorage[pm.defaultUserKey]);
    },
    
    /**
     * Unlink user from client
     * @param {String} userKey
     * @returns {deferred}
     */
    unlinkUser: function (userKey) {
        var deferred = $.Deferred();
        localStorage.removeItem(userKey);
        if (app.isMoble) {
            // delete 
            deferred.resolve();
        } else {
            deferred.resolve();
        }
        return deferred;
    },
    
    /**
     * Return linked user list
     * @returns {Array}
     */
    getUserList: function () {
        var list = localStorage[pm.userListKey];
        if (! list) {
            list = new Array();
        } else {
            list = JSON.parse(list);
        }
        return list;
    },
    
    /**
     * Save user list
     * @param {Array} list
     * @returns {undefined}
     */
    saveUserList: function (list) {
        var jsonList = JSON.stringify(list);
        console.log(jsonList);
        localStorage[pm.userListKey] = JSON.stringify(list);
    },
    
    /**
     * Link user to the client application
     * @param {type} profile
     * @returns {Promise} Promise Object
     */
    linkUser: function (profile) {
        var deferred = $.Deferred();
        var prevProfile = pm.searchLocalUserLinked(profile);
        if (false !== prevProfile) {
            deferred.reject('El perfil indicado ya estaba enlazado previamente');
        } else {
            var list = pm.getUserList();

            var user = {
                name: profile.name,
                lastName: profile.lastname,
                key: profile.key
            };
            
            // TODO Get Identification with image from server
            list.push(user);
            pm.saveUserList(list);
            deferred.resolve();
        }
        return deferred.promise();
    },
    
    /**
     * Search a user on users list
     * @param {String} key User key
     * @returns {profile|false}
     */
    searchLocalUserLinked: function (key) {
        var list = pm.getUserList();
        for (var i = 0; i<list.length; i++) {
            if (list[i].key == key) {
                return list[i];
            }
        }
        return false;
    },
    
    /**
     * Perform login request to the server
     * @param {type} user
     * @param {type} password
     * @returns {unresolved}
     */
    login: function login(user, password) {
        var deferred = $.Deferred();
        var processRequest = function (data, textStatus, jqXHR) {
                                console.log(arguments);
                                if (data.result == 'OK') {
                                    pm.linkUser(data.profile)
                                            .done(function () {deferred.resolve();})
                                            .fail(function (msg) {deferred.reject(msg);});
                                } else {
                                    var msg = '';
                                    switch (data.result) {
                                        case 'INVALID DATA': 
                                            msg = '';
                                            for (key in data.messages) {
                                                for (error in data.messages[key]) {
                                                    msg+= data.messages[key][error] + '<br>';
                                                }
                                            }
                                            break;
                                        case 'LOGIN INCORRECT': 
                                            msg = 'Usuario o contraseña incorrecta';
                                            break;
                                    }
                                    deferred.reject(msg);
                                }
                            };
        var failRequest = function (jqXHR, data) {
                                console.log(data);
                                deferred.reject('No se puede conectar con el servidor');
                            };
        app.serverRequest(pm.server.login,{ mail: user, password: password}).then (processRequest,failRequest);
        //app.loadServerPage('profile/login', '#wrapper').fail(function () {
        return deferred.promise();
    },
    
    /**
     * Show login form
     * @param {String} user
     * @param {String} password
     * @returns {undefined}
     */
    showLoginFormAction: function (user, password) {
        user = user || '';
        password = password || '';
        var form = app.loadLocalPage(pm.localForms.login);
        $(form).bind('loginClick',function (event) {
            popup = event.delegateTarget;
            event.stopImmediatePropagation();
            user = $(popup).find('#username').val();
            pass = $(popup).find('#password').val();
            if (! user) {
                app.alertMessage('Por favor, introduce un nombre de usuario');
            } else if (! pass) {
                app.alertMessage('Por favor, introduce una contraseña');
            } else {
                pm.login(user,pass)
                        .done (function () {
                            window.history.back();
                        })
                        .fail (function (message) {
                            console.log('Login error: ' + message);
                            app.message(message,'ui-icon-alert ui-content ui-btn-icon-left');
                        });
            }
            return false;
        });
    },
    
    /**
     * Populate user list
     * @param {type} list
     * @returns {undefined}
     */
    populateUserList: function (list) {
        var users = pm.getUserList();
        list.empty();
        for (i=0 ; i < users.length ; i++) {
            list.append('<li class="ui-li-static ui-btn-icon-right ui-icon-carat-r" data-key="'+users[i].key+'">'+ users[i].name + ' ' + users[i].lastName + '</li>');
        }
    },
    
    /**
     * Create and show user list
     * @param {method} callBack function to success 
     * @returns {undefined}
     */
    showUserListAction: function(callback) {
        var form = app.loadLocalPage(pm.localForms.users);
        $(form).bind('listClick', function (event, data) {
            pm.setDefaultUser(data);
            callback(pm.searchLocalUserLinked(data));
        });
    }
}