<?php

/* 
 * Copyright (C) 2013 Rafael Barragán Górriz
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Profile\Controller;

use Zend\Mvc\Controller\AbstractActionController;
use Zend\View\Model\ViewModel;
use Zend\View\Model\JsonModel;
use Zend\Validator\EmailAddress;
use Zend\Validator\NotEmpty;
use Zend\InputFilter\InputFilter;
use Profile\Model\Profile;          
use Profile\Form\ProfileForm;
use Profile\Form\LoginForm;
use Application\Model\Application;

class ProfileController extends AbstractActionController
{
    protected $profileTable;
    
    public function indexAction()
    {
        return new ViewModel(array(
             'profiles' => $this->getProfileTable()->fetchAll(),
        ));
    }

    public function addAction()
    {
        $form = new ProfileForm();
        $form->get('submit')->setValue('Add');

        $request = $this->getRequest();
        if ($request->isPost()) {
            $profile = new Profile();
            $profile->setAdapter($this->getProfileTable()->getTableGateway()->getAdapter());
            $form->setInputFilter($profile->getInputFilter());
            $form->setData($request->getPost());

            if ($form->isValid()) {
                $profile->exchangeArray($form->getData());
                // Adding unique key
                $profile->key = sprintf('%u', ip2long($request->getServer()->get('REMOTE_ADDR'))) . floor(microtime(true) * 1000);
                Application::getLogger()->info($profile);
                $this->getProfileTable()->saveProfile($profile);
                
                // Creating profile path
                $this->createProfilePath($profile);
                
                // Return user key and response
                $response = $this->getResponse();
                
                $response->getHeaders()->addHeaderLine('Content-Type', 'application/json');
                
                $data = json_encode(array('result' => 'OK', 'profile' => $profile));
                $response->setContent($data);
                return $response;
            }
        }
        return array('form' => $form);
    }

    public function editAction()
    {
        $id = (int) $this->params()->fromRoute('id', 0);
        if (!$id) {
            return $this->redirect()->toRoute('profile', array(
                'action' => 'add'
            ));
        }

        // Get the Album with the specified id.  An exception is thrown
        // if it cannot be found, in which case go to the index page.
        try {
            $profile = $this->getProfileTable()->getProfile($id);
        }
        catch (\Exception $ex) {
            return $this->redirect()->toRoute('profile', array(
                'action' => 'index'
            ));
        }

        $form  = new ProfileForm();
        $form->bind($profile);
        $form->get('submit')->setAttribute('value', 'Edit');

        $request = $this->getRequest();
        if ($request->isPost()) {
            $profile->setAdapter($this->getProfileTable()->getTableGateway()->getAdapter());
            $form->setInputFilter($profile->getInputFilter());
            $form->setData($request->getPost());

            if ($form->isValid()) {
                $this->getProfileTable()->saveProfile($profile);

                // Redirect to list of albums
                return $this->redirect()->toRoute('profile');
            }
        }

        return array(
            'id' => $id,
            'form' => $form,
        );
    }

    public function deleteAction()
    {
        $id = (int) $this->params()->fromRoute('id', 0);
        if (!$id) {
            return $this->redirect()->toRoute('profile');
        }

        $request = $this->getRequest();
        if ($request->isPost()) {
            $del = $request->getPost('del', 'No');

            if ($del == 'Yes') {
                $id = (int) $request->getPost('id');
                $this->getProfileTable()->deleteProfile($id);
            }

            // Redirect to list of albums
            return $this->redirect()->toRoute('profile');
        }

        return array(
            'id'    => $id,
            'profile' => $this->getProfileTable()->getProfile($id)
        );
    }
    
    public function loginAction () {
        $form = new LoginForm();
        $form->get('submit')->setValue('Login');

        $request = $this->getRequest();
        if ($request->isPost()) {
                      
            $inputFilter = new InputFilter();
            
            $inputFilter->add(array(
                'name'     => 'mail',
                'required' => true,
                'filters'  => array(
                    array('name' => 'StripTags'),
                    array('name' => 'StringTrim'),
                ),
                'validators' => array(
                    array(
                        'name' => 'NotEmpty',
                        'options' => array(
                            'messages' => array(NotEmpty::IS_EMPTY => 'Introduce un nombre de usuario')
                        ),
                    ),
                    array (
                       'name'    => 'EmailAddress',
                       'options' => array(
                           'messages' => array(EmailAddress::INVALID_FORMAT => 'Recuerda que el nombre de usuario debe ser una dirección de correo válida')
                       ),
                    ),
                ),
            ));

            $inputFilter->add(array(
                'name'     => 'password',
                'required' => true,
                'filters'  => array(
                    array('name' => 'StripTags'),
                    array('name' => 'StringTrim'),
                ),
                'validators' => array(
                    array(
                        'name' => 'NotEmpty',
                        'options' => array(
                            'messages' => array(NotEmpty::IS_EMPTY => 'Introduce una contraseña')
                        ),
                    ),
                )
            ));
            
            $form->setInputFilter($inputFilter);
            $form->setData($request->getPost());
            if (! $form->isValid()) {
                return new JsonModel(array('result' => 'INVALID DATA', 'messages' => $form->getMessages()));
            } else {
                $mail = $form->getInputFilter()->getValue('mail');
                $password = $form->getInputFilter()->getValue('password');
                // Try to get user key
                $profile = $this->getProfileTable()->login($mail,$password);
                
                if ($profile === false) {
                    return new JsonModel(array('result' => 'LOGIN INCORRECT'));
                    /*$form->setMessages(array('mail' => array('Login incorrect')));
                    return array('form' => $form);*/
                } else {
                    // Return user key and response
                    $response = $this->getResponse();

                    $response->getHeaders()->addHeaderLine('Content-Type', 'application/json');

                    $data = json_encode(array('result' => 'OK', 'profile' => $profile));
                    $response->setContent($data);
                    return $response;
                }
            }
        }
        return array('form' => $form);
    }
    
    public function getProfileTable()
    {
        if (!$this->profileTable) {
            $sm = $this->getServiceLocator();
            $this->profileTable = $sm->get('Profile\Model\ProfileTable');
        }
        return $this->profileTable;
    }

    private function createProfilePath ($profile) {
        $userDir = Application::getDataDir($profile->key);
        mkdir($userDir,0,true);
        $media = Application::getMediaDir($profile->key);
        mkdir($media);
    }
    
    private function createDefaultBoard ($profile) {
        
    }
}